/*=========================================================================

  Program:   Igns global tools
  Module:    $RCSfile: doublewindowfileviewer.cpp,v $
  Language:  C++
  Date:      $Date: 2009-05-05 18:35:52 $
  Version:   $Revision: 1.6 $

  Copyright (c) 2007-2009 IPL, BIC, MNI, McGill, Anka
  All rights reserved.
  
=========================================================================*/
#include <qstring.h>
#include <qfiledialog.h>
#include <qdir.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include "ignsmsg.h"
#include "vtkTransform.h"
#include "vtkRenderer.h"
#include "vtkCamera.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkInteractorStyleImage.h"
#include "vtkLookupTable.h"
#include "vtkImageShiftScale.h"
#include "vtkImageMapToWindowLevelColors.h"
#include "vtkImageReslice.h"
#include "vtkImageData.h"
#include "vtkImageActor.h"
#include "vtkImagePlaneActor.h"
#include "vtkProperty.h"
#include "vtkOutlineFilter.h"
#include "vtkPolyDataMapper.h"
#include "doublewindowfileviewer.h"
#include "vtkImagePlaneActor.h"

DoubleWindowFileViewer::DoubleWindowFileViewer( QWidget* parent, const char* name, WFlags fl )
    : DoubleViewWindow( parent, name, fl ),
    m_initLeftViewDone(0),
    m_leftImageActor(0)
{
    this->InitializeActors();
    m_imageTransform = vtkTransform::New();
    m_imageTransform->Identity();
    m_inputResliceTransform = vtkTransform::New();
    m_inputResliceTransform->Identity();
}

DoubleWindowFileViewer::~DoubleWindowFileViewer()
{
    if (m_leftImageActor)
        m_leftImageActor->Delete();
    if (m_inputResliceTransform)
        m_inputResliceTransform->Delete();
}

void DoubleWindowFileViewer::InitializeActors()
{    
    m_usOutlineActor = 0;
    m_usOutlineMapper = 0;
    m_usOutlineFilter = 0;
    
    m_mriOutlineActor = 0;
    m_mriOutlineMapper = 0;
    m_mriOutlineFilter = 0;
    
    m_usActor = 0;
    m_leftImageActor = vtkImagePlaneActor::New();
    m_mriActor = vtkImageActor::New();
    m_mriActor->InterpolateOff();
}

void DoubleWindowFileViewer::InitLeftView()
{
    m_usRenderer->AddViewProp( m_leftImageActor );
    m_usInteractor->Initialize();
    vtkInteractorStyleImage * style = vtkInteractorStyleImage::New();
    m_usInteractor->SetInteractorStyle( style );
    style->Delete();
    m_initLeftViewDone = 1;
}

void DoubleWindowFileViewer::SetUltrasoundImage(vtkImageData *image)
{
    m_leftImageActor->SetInput(image);
    if (!m_initLeftViewDone)
        this->InitLeftView();
    this->UpdateViews();
}

void DoubleWindowFileViewer::SetLeftDefaultView()
{
    vtkImageData *input = m_leftImageActor->GetInput();
    if (input)
    {
        input->UpdateInformation();
        int * extent = input->GetWholeExtent();
        int diffx = extent[1] - extent[0] + 1;
        vtkReal scalex = (vtkReal)diffx / 2.0; 
        int diffy = extent[3] - extent[2] + 1;
        vtkReal scaley = (vtkReal)diffy / 2.0;
        
        vtkCamera * cam = m_usRenderer->GetActiveCamera();
        cam->ParallelProjectionOn();
        cam->SetParallelScale( scaley );
        double * prevPos = cam->GetPosition();
        double * prevFocal = cam->GetFocalPoint();
        cam->SetPosition( scalex, scaley, prevPos[2] );
        cam->SetFocalPoint( scalex, scaley, prevFocal[2] );
    }
    m_usRenderer->Render();
}

void DoubleWindowFileViewer::SetResliceTransform(vtkMatrix4x4 *dc)
{
   m_reslice->SetResliceAxes(dc);
}

void DoubleWindowFileViewer::SetResliceTransform(vtkTransform *resTransform)
{
    m_inputResliceTransform->Identity();
    m_inputResliceTransform->Concatenate(resTransform->GetMatrix());
#if !defined(WIN32) && !defined(_WIN32)
    vtkMatrix4x4 *mat = m_imageTransform->GetMatrix();
    igstkLogMacro( DEBUG, "DoubleWindowFileViewer m_imageTransform:" << "\n" <<
    mat->Element[0][0] << ", " <<
    mat->Element[0][1] << ", " <<
    mat->Element[0][2] << ", " <<
    mat->Element[0][3] << "\n" <<
    mat->Element[1][0] << ", " <<
    mat->Element[1][1] << ", " <<
    mat->Element[1][2] << ", " <<
    mat->Element[1][3] << "\n" <<
    mat->Element[2][0] << ", " <<
    mat->Element[2][1] << ", " <<
    mat->Element[2][2] << ", " <<
    mat->Element[2][3] <<
    endl);
    mat = m_inputResliceTransform->GetMatrix();
    igstkLogMacro( DEBUG, "DoubleWindowFileViewer m_inputResliceTransform:" << "\n" <<
    mat->Element[0][0] << ", " <<
    mat->Element[0][1] << ", " <<
    mat->Element[0][2] << ", " <<
    mat->Element[0][3] << "\n" <<
    mat->Element[1][0] << ", " <<
    mat->Element[1][1] << ", " <<
    mat->Element[1][2] << ", " <<
    mat->Element[1][3] << "\n" <<
    mat->Element[2][0] << ", " <<
    mat->Element[2][1] << ", " <<
    mat->Element[2][2] << ", " <<
    mat->Element[2][3] <<
    endl);
#endif
}

void DoubleWindowFileViewer::RestoreViewsButtonClicked()
{
    this->SetLeftDefaultView();
    this->SetDefaultView(m_mriActor, m_mriRenderer);
    this->RestoreGridAndBlending();
    this->UpdateViews();
}

void DoubleWindowFileViewer::SetImageData(vtkImageData *data)
{
    if( m_imageData == data )
    {
        return;
    }
    
    if( m_imageData )
    {
        m_imageData->UnRegister( 0 );
    }
    
    m_imageData = data;
    
    if( m_imageData )
    {
        m_imageData->Register( 0 );

#if !defined(WIN32) && !defined(_WIN32)
        vtkMatrix4x4 *mat = m_imageTransform->GetMatrix();
        igstkLogMacro( DEBUG, "DoubleWindowFileViewer m_imageTransform:" << "\n" <<
        mat->Element[0][0] << ", " << 
        mat->Element[0][1] << ", " <<
        mat->Element[0][2] << ", " << 
        mat->Element[0][3] << "\n" << 
        mat->Element[1][0] << ", " << 
        mat->Element[1][1] << ", " <<
        mat->Element[1][2] << ", " <<
        mat->Element[1][3] << "\n" <<
        mat->Element[2][0] << ", " << 
        mat->Element[2][1] << ", " <<
        mat->Element[2][2] << ", " <<
        mat->Element[2][3] << 
        endl);
#endif
        vtkTransform * concat = vtkTransform::New();
        concat->Identity();
        concat->SetInput( m_imageTransform->GetLinearInverse() );
        concat->Concatenate( m_inputResliceTransform );
        
        vtkReal range[2];
        m_imageData->GetScalarRange( range );
        m_win_level[0] = range[1] - range[0];
        m_win_level[1] = 0.5 * (range[1] - range[0]);
        
        vtkLookupTable *lut = vtkLookupTable::New(); // lookup table needed for blending
        lut->SetTableRange(0, 255); 
        lut->SetHueRange( 1, 1 );
        lut->SetSaturationRange( 0, 0 );
        lut->SetValueRange( 1, 1 );
        lut->Build();

        m_displayWindowLevelColorsImage = vtkImageMapToWindowLevelColors::New();
        int scalarType = m_imageData->GetScalarType();
        if (scalarType != VTK_UNSIGNED_CHAR)
        { 
            m_displayImage = vtkImageShiftScale::New();
            m_displayImage->SetInput(m_imageData);
            m_displayImage->SetScale(defaultScale); //TODO - compute correct scale
            m_displayImage->SetOutputScalarTypeToUnsignedChar();

            m_displayWindowLevelColorsImage->SetInput(m_displayImage->GetOutput());
            m_displayWindowLevelColorsImage->SetWindow(m_win_level[0]*defaultScale);
            m_displayWindowLevelColorsImage->SetLevel(m_win_level[1]*defaultScale);
        }
        else
        {
            m_displayWindowLevelColorsImage->SetInput(m_imageData);
            m_displayWindowLevelColorsImage->SetWindow(m_win_level[0]);
            m_displayWindowLevelColorsImage->SetLevel(m_win_level[1]);
        }       
        m_displayWindowLevelColorsImage->SetLookupTable(lut);

        m_reslice->SetInput( m_displayWindowLevelColorsImage->GetOutput() );
        m_reslice->SetInterpolationModeToLinear( );
        m_reslice->SetOutputExtent(0, 639, 0, 479, 0, 1);
        m_reslice->SetOutputOrigin(0, 0, 0);
        m_reslice->SetOutputDimensionality(2);
        m_reslice->SetOutputSpacing(1,1,1);
//        m_reslice->SetResliceTransform(m_inputResliceTransform);
        m_reslice->SetResliceTransform(concat);
        m_reslice->UpdateInformation();
        
        m_mriActor->SetInput(m_reslice->GetOutput());
        m_mriRenderer->AddViewProp( m_mriActor );
        this->SetDefaultView(m_mriActor,m_mriRenderer );
        m_mriInteractor->Initialize();
        vtkInteractorStyleImage * style = vtkInteractorStyleImage::New();
        m_mriInteractor->SetInteractorStyle( style );
        style->Delete();
        lut->Delete();
        concat->Delete();
    }
}

void DoubleWindowFileViewer::SetRightImageWindow(int window)
{
    m_win_level[0] = window;
    m_displayWindowLevelColorsImage->SetWindow(m_win_level[0]);
    this->UpdateViews();
}

void DoubleWindowFileViewer::SetRightImageLevel(int level)
{
    m_win_level[1] = level;
    m_displayWindowLevelColorsImage->SetLevel(m_win_level[1]);
    this->UpdateViews();
}

void DoubleWindowFileViewer::SetRightImageColor(vtkLookupTable *lut)
{
    m_displayWindowLevelColorsImage->SetLookupTable(lut);
    this->UpdateViews();
}

void DoubleWindowFileViewer::SetResliceSpacing(double stepX, double stepY, double stepZ)
{
    m_reslice->SetOutputSpacing(stepX, stepY, stepZ);
}

void DoubleWindowFileViewer::SetRightImageTransform(vtkTransform *t)
{
    m_imageTransform->Identity();
    m_imageTransform->Concatenate(t->GetMatrix());
#if !defined(WIN32) && !defined(_WIN32)
        vtkMatrix4x4 *mat = t->GetMatrix();
        igstkLogMacro( DEBUG, "DoubleWindowFileViewer input:" << "\n" <<
        mat->Element[0][0] << ", " << 
        mat->Element[0][1] << ", " <<
        mat->Element[0][2] << ", " << 
        mat->Element[0][3] << "\n" << 
        mat->Element[1][0] << ", " << 
        mat->Element[1][1] << ", " <<
        mat->Element[1][2] << ", " <<
        mat->Element[1][3] << "\n" <<
        mat->Element[2][0] << ", " << 
        mat->Element[2][1] << ", " <<
        mat->Element[2][2] << ", " <<
        mat->Element[2][3] << 
        endl);
        mat = m_imageTransform->GetMatrix();
        igstkLogMacro( DEBUG, "DoubleWindowFileViewer m_imageTransform:" << "\n" <<
        mat->Element[0][0] << ", " << 
        mat->Element[0][1] << ", " <<
        mat->Element[0][2] << ", " << 
        mat->Element[0][3] << "\n" << 
        mat->Element[1][0] << ", " << 
        mat->Element[1][1] << ", " <<
        mat->Element[1][2] << ", " <<
        mat->Element[1][3] << "\n" <<
        mat->Element[2][0] << ", " << 
        mat->Element[2][1] << ", " <<
        mat->Element[2][2] << ", " <<
        mat->Element[2][3] << 
        endl);
#endif
}

