/*=========================================================================

  Program:   Igns global tools
  Module:    $RCSfile: multiplefileviewerdialog.cpp,v $
  Language:  C++
  Date:      $Date: 2009-06-16 13:51:49 $
  Version:   $Revision: 1.11 $

  Copyright (c) 2007-2009 IPL, BIC, MNI, McGill, Anka
  All rights reserved.

=========================================================================*/

#include "multiplefileviewerdialog.h"

#include <sstream>
#include <stdlib.h>

#include <qpushbutton.h>
#include <qslider.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qdir.h>
#include <qtimer.h>
#include <qfileinfo.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include "ignsmsg.h"
#include "doublewindowfileviewer.h"
#include "scenemanager.h"
#include "vtkMINCImageReader.h"
#include "vtkTransform.h"
#include "vtkQtMatrixDialog.h"
#include "vtkMatrix4x4.h"
#include "vtkImageData.h"
#include "vtkMINCImageAttributes.h"
#include "vtkMINC.h"
#include "mincinfowidget.h"
#include "yesnodialog.h"

AcquisitionData::AcquisitionData():
    m_imageData(0),
    m_transform(0),
    m_startX(0.0),
    m_startY(0.0),
    m_startZ(0.0),
    m_directionCosines(0)
{
}

AcquisitionData::~AcquisitionData()
{
    if (m_transform)
        m_transform->Delete();
    if (m_directionCosines)
        m_directionCosines->UnRegister(0);        
}

void AcquisitionData::SetImage( vtkImageData * image )
{
    if( m_imageData == image )
    {
        return;
    }

    if( m_imageData )
    {
        m_imageData->UnRegister( 0 );
    }
    m_imageData = image;
    if( m_imageData )
    {
        m_imageData->Register( 0 );
    }
}

void AcquisitionData::SetTransform(vtkTransform * trans)
{
    if( !m_transform )
    {
        m_transform = vtkTransform::New();
        m_transform->Identity();
    }
    if( m_transform && trans)
    {
        m_transform->Concatenate(trans->GetMatrix());
    }
}

void AcquisitionData::SetStartCoordinates(double x, double y, double z)
{
    m_startX = x;
    m_startY = y;
    m_startZ = z;
}

void AcquisitionData::GetStartCoordinates(double *x, double *y, double *z)
{
    *x = m_startX;
    *y = m_startY;
    *z = m_startZ;
}

void AcquisitionData::SetStep(double x, double y, double z)
{
    m_stepX = x;
    m_stepY = y;
    m_stepZ = z;
}

void AcquisitionData::GetStep(double *x, double *y, double *z)
{
    *x = m_stepX;
    *y = m_stepY;
    *z = m_stepZ;
}

void AcquisitionData::SetDirectionCosines(vtkMatrix4x4 *dc)
{
    if( m_directionCosines == dc )
    {
        return;
    }

    if( m_directionCosines )
    {
        m_directionCosines->UnRegister( 0 );
    }
    m_directionCosines = dc;
    if( m_directionCosines )
    {
        m_directionCosines->Register( 0 );
    }
}

vtkMatrix4x4 * AcquisitionData::GetDirectionCosines()
{
    return m_directionCosines;
}

MultipleFileViewerDialog::MultipleFileViewerDialog( QWidget* parent, const char* name, WFlags fl )
    : MultipleFileViewerDialogBase( parent, name, fl ),
    m_viewer(0),
    m_manager(0),
    m_acquisitionImageCounter(0),
    m_timeInterval(DEFAULT_TIME_INTERVAL * 1000),
    m_slideShowOn(false),
    m_current_index(0),
    m_discardedFilesDirectory("")
{
    if ( !name )
        setName( "MultipleFileViewerDialog" );
        
    m_timer = new QTimer( NULL );     
    connect( m_timer, SIGNAL(timeout()), this, SLOT(SetNextImage()) );
}

MultipleFileViewerDialog::~MultipleFileViewerDialog()
{
    this->CleanAcquisition();
    if (m_manager != NULL) 
    { 
        m_manager->UnRegister(0); 
    } 
    delete  m_timer;
}

void MultipleFileViewerDialog::LoadFilesButtonClicked()
{
    QStringList inputFiles;
    QString  filename;
    vtkMINCImageAttributes *attributes;// = vtkMINCImageAttributes::New();
    double x, y, z;
    if (m_acquisitionImageCounter > 0)
        this->CleanAcquisition();
    inputFiles = QFileDialog::getOpenFileNames("Minc file (*.mnc)",
                                m_lastVisitedDirectory,
                                0,
                                "open",
                                "Read Frames");
    for ( QStringList::Iterator it = inputFiles.begin(); it != inputFiles.end(); ++it )
    {
        filename = (*it);
        vtkMINCImageReader * reader = vtkMINCImageReader::New();
        if( reader->CanReadFile( filename ) )
        {
            AcquisitionData *data = AcquisitionData::New();
            reader->SetFileName(filename);
            reader->Update();
            attributes = reader->GetImageAttributes();
            x = attributes->GetAttributeValueAsDouble(MIxspace, MIstart);
            y = attributes->GetAttributeValueAsDouble(MIyspace, MIstart);
            z = attributes->GetAttributeValueAsDouble(MIzspace, MIstart);
            data->SetFileName(filename);
            data->SetImage(reader->GetOutput());
            data->SetStartCoordinates(x, y, z);
            x = attributes->GetAttributeValueAsDouble(MIxspace, MIstep);
            y = attributes->GetAttributeValueAsDouble(MIyspace, MIstep);
            z = attributes->GetAttributeValueAsDouble(MIzspace, MIstep);
            data->SetStep(x, y, z);
            data->SetDirectionCosines(reader->GetDirectionCosines());
            m_acquisition.push_back( data );
        }
        //TODO else {} error message
        reader->Delete();
    }
    m_acquisitionImageCounter = m_acquisition.size();
    if (m_acquisitionImageCounter > 0)
    {
        framesSlider->setMaxValue(m_acquisitionImageCounter-1);
        m_viewer->SetUltrasoundImage(m_acquisition[0]->GetImage());
        vtkTransform *trans = vtkTransform::New();
        this->ComputeResliceTransform(0, trans);
        m_viewer->SetResliceTransform(trans);
        trans->Delete();
/*        m_acquisition[0]->GetStep(&x, &y, &z);
        m_viewer->SetResliceTransform(m_acquisition[0]->GetDirectionCosines());
        m_viewer->SetResliceSpacing(x, y, z);*/
        filenameLineEdit->setText(m_acquisition[0]->GetFileName());
        m_acquisition[0]->GetStartCoordinates(&x, &y, &z);
        startXLineEdit->setText(QString::number(x));
        startYLineEdit->setText(QString::number(y));
        startZLineEdit->setText(QString::number(z));
        m_current_index = 0;
        m_viewer->SetLeftDefaultView();
    }
}

void MultipleFileViewerDialog::LoadVolumeButtonClicked()
{
    if (m_lastVisitedDirectory.isEmpty())
        m_lastVisitedDirectory = QDir::homeDirPath();

    QString filename = QFileDialog::getOpenFileName( m_lastVisitedDirectory, "Minc file (*.mnc)", 0 , "open", "Open minc file" ); 
    if( !filename.isNull() )
    {
        vtkImageData *image = 0;
        if( QFile::exists( filename ) )
        {
            vtkMINCImageReader * reader = vtkMINCImageReader::New();
            reader->SetFileName( filename );
        
            reader->Update();
            image = reader->GetOutput();
            m_viewer->SetImageData(image);
            reader->Delete();
            QFileInfo info( filename );
            m_lastVisitedDirectory = info.dirPath( true );
        }
        else
            QMessageBox::warning( this, "Error: can't open file: ", filename, 1, 0 );
        vtkReal range[2];
        image->GetScalarRange(range);
        rightWindowSlider->blockSignals(true);
        rightLevelSlider->blockSignals(true);
        rightWindowSlider->setMaxValue( (int)(range[1]-range[0]) ); 
        rightWindowSlider->setLineStep((int)(0.02*(range[1] - range[0]))); //50 steps is enough 
        rightLevelSlider->setMaxValue( (int)((range[1]-range[0])/2) ); 
        rightLevelSlider->setLineStep((int)(0.04*(range[1] - range[0]))); //50 steps is enough 
        rightWindowSlider->setValue(rightWindowSlider->maxValue());
        rightLevelSlider->setValue(rightLevelSlider->maxValue());
        rightWindowSlider->blockSignals(false);
        rightLevelSlider->blockSignals(false);
    }
}

void MultipleFileViewerDialog::DeleteCurrentButtonClicked()
{
    AcquisitionDataList::iterator it = std::find( m_acquisition.begin(), m_acquisition.end(), m_acquisition[m_current_index] );
    if( it != m_acquisition.end() )
    {
        QFileInfo info( m_acquisition[m_current_index]->GetFileName() );
        QString initialPath = info.dirPath( true );
        QString fileName = info.fileName();
        if (m_discardedFilesDirectory.isEmpty())
        {
            char tempName[512];
            sprintf(tempName, "%s/%s", initialPath.ascii(), DISCARDED_ACQUISITIONS_DIR);
            if (mkdtemp(tempName))
            {
                m_discardedFilesDirectory = tempName;
                QString infoMsg = "Discarded files will be moved to " + m_discardedFilesDirectory;
                QMessageBox::information( 0, "Moving Files", infoMsg, 1, 0 );
            }
        }
        if (!m_discardedFilesDirectory.isEmpty())
        {
            QString newName(m_discardedFilesDirectory);
            newName.append("/");
            newName.append(fileName);
            if (rename((const char*)(m_acquisition[m_current_index]->GetFileName()), (const char*)newName) != 0)
            {
                QString  accessError = IGNS_MSG_CANT_REMOVE + newName;
                QMessageBox::warning( 0, "Warning", accessError, 1, 0 );
            }
        }
        else
        {
            YesNoDialog *dlg = new YesNoDialog(0, "removeFile", TRUE, Qt::WDestructiveClose );
            QString tmp = "File " + m_acquisition[m_current_index]->GetFileName() +
                        "\ncannot be moved to e temporary directory.\nDelete immediately?";
            dlg->SetQuestion(tmp);
            if (dlg->exec() == QDialog::Accepted)
            {
                std::remove(m_acquisition[m_current_index]->GetFileName());
                m_acquisition[m_current_index]->Delete();
            }
        }
        m_acquisition.erase( it );
        m_acquisitionImageCounter = m_acquisition.size();
        framesSlider->setMaxValue(m_acquisitionImageCounter-1);
        FrameSliderValueChanged(m_current_index);
    }
}

void MultipleFileViewerDialog::FirstButtonClicked()
{
    m_current_index = 0;
    framesSlider->setValue(0);
}

void MultipleFileViewerDialog::PreviousButtonClicked()
{
    if  (m_current_index > 0)
    {
        m_current_index--;
        framesSlider->setValue(m_current_index);
    }
}

void MultipleFileViewerDialog::FrameSliderValueChanged(int index)
{
    double x, y, z;
    m_current_index = index;
    if (m_acquisitionImageCounter > 0)
    {
        m_viewer->SetUltrasoundImage(m_acquisition[m_current_index]->GetImage());
        vtkTransform *trans = vtkTransform::New();
        this->ComputeResliceTransform(index, trans);
        m_viewer->SetResliceTransform(trans);
        trans->Delete();
/*        m_viewer->SetResliceTransform(m_acquisition[m_current_index]->GetDirectionCosines());*/
        filenameLineEdit->setText(m_acquisition[m_current_index]->GetFileName());
        m_acquisition[m_current_index]->GetStartCoordinates(&x, &y, &z);
        startXLineEdit->setText(QString::number(x));
        startYLineEdit->setText(QString::number(y));
        startZLineEdit->setText(QString::number(z));
        m_viewer->SetLeftDefaultView();
    }
}

void MultipleFileViewerDialog::SetNextImage()
{
    if  (m_current_index < m_acquisitionImageCounter-1)
    {
        m_current_index++;
        framesSlider->setValue(m_current_index);
    }
    else
    {
        m_timer->stop();
        m_slideShowOn = !m_slideShowOn;
        startPushButton->setText( "Start" );
        this->SetButtonsEnabled(true);
    }
}

void MultipleFileViewerDialog::TimeComboBoxActivated(int timeInt)
{
    m_timeInterval = (timeInt + SHORTEST_TIME_INTERVAL)*1000;
}

void MultipleFileViewerDialog::StartButtonClicked()
{
    if (m_slideShowOn)
    {
        m_timer->stop();
        startPushButton->setText( "Start" );
        this->SetButtonsEnabled(true);
    }
    else
    {
        if  (m_current_index == m_acquisitionImageCounter-1)
        {
            m_current_index = 0;
            framesSlider->setValue(0);
        }
        m_timer->start( m_timeInterval, false );
        startPushButton->setText( "Stop" );
        this->SetButtonsEnabled(false);
    }
    m_slideShowOn = !m_slideShowOn;
}

void MultipleFileViewerDialog::SetTransformButtonClicked()
{
    vtkTransform *registrationTransform = vtkTransform::New();
    registrationTransform->Identity();
    vtkMatrix4x4 *tmpMat;
/*    if (m_viewer->GetTransform())
    {
        tmpMat = m_viewer->GetTransform()->GetMatrix();
        registrationTransform->Concatenate(tmpMat);
    }
    else
    {
        tmpMat = vtkMatrix4x4::New();
        tmpMat->Identity();
        registrationTransform->Concatenate(tmpMat);
        tmpMat->Delete();
    }*/
    tmpMat = vtkMatrix4x4::New();
    vtkQtMatrixDialog * dlg;
/*    tmpMat = registrationTransform->GetMatrix();*/
    dlg = new vtkQtMatrixDialog( tmpMat, false, 0, "" );
    if (dlg->exec() == QDialog::Accepted)
    {
        registrationTransform->Concatenate(tmpMat);
        m_viewer->SetRightImageTransform(registrationTransform);
#if !defined(WIN32) && !defined(_WIN32)
        vtkMatrix4x4 *mat = registrationTransform->GetMatrix();
        igstkLogMacro( DEBUG, "MultipleFileViewerDialog registrationTransform:" << "\n" <<
        mat->Element[0][0] << ", " << 
        mat->Element[0][1] << ", " <<
        mat->Element[0][2] << ", " << 
        mat->Element[0][3] << "\n" << 
        mat->Element[1][0] << ", " << 
        mat->Element[1][1] << ", " <<
        mat->Element[1][2] << ", " <<
        mat->Element[1][3] << "\n" <<
        mat->Element[2][0] << ", " << 
        mat->Element[2][1] << ", " <<
        mat->Element[2][2] << ", " <<
        mat->Element[2][3] << 
        endl);
#endif
        registrationTransform->Delete();
        tmpMat->Delete();
    }
}

void MultipleFileViewerDialog::NextButtonClicked()
{
    if  (m_current_index < m_acquisitionImageCounter-1)
    {
        m_current_index++;
        framesSlider->setValue(m_current_index);
    }
}

void MultipleFileViewerDialog::LastButtonClicked()
{
    m_current_index = m_acquisitionImageCounter-1;
    framesSlider->setValue(m_current_index);
}

void MultipleFileViewerDialog::LeftWindowSliderValueChanged(int)
{
    qWarning( "MultipleFileViewerDialog::LeftWindowSliderValueChanged(int): Not implemented yet" );
}

void MultipleFileViewerDialog::LeftLevelSliderValueChanged(int)
{
    qWarning( "MultipleFileViewerDialog::LeftLevelSliderValueChanged(int): Not implemented yet" );
}

void MultipleFileViewerDialog::LeftColorComboBoxActivated(int)
{
    qWarning( "MultipleFileViewerDialog::LeftColorComboBoxActivated(int): Not implemented yet" );
}

void MultipleFileViewerDialog::RightWindowSliderValueChanged(int window)
{
    m_viewer->SetRightImageWindow(window);
}

void MultipleFileViewerDialog::RightLevelSliderValueChanged(int level)
{
    m_viewer->SetRightImageLevel(level);
}

void MultipleFileViewerDialog::RightColorComboBoxActivated(int index)
{
    static vtkLookupTable *lut = vtkLookupTable::New();
    vtkReal range[2] = {0, 255};
    QString name = m_manager->GetTemplateLookupTableName(index);
    m_manager->CreateLookupTable(name, range, lut, false);
    rightWindowSlider->blockSignals(true);
    rightLevelSlider->blockSignals(true);
    rightWindowSlider->setValue(rightWindowSlider->maxValue());
    rightLevelSlider->setValue(rightLevelSlider->maxValue());
    rightWindowSlider->blockSignals(false);
    rightLevelSlider->blockSignals(false);
    m_viewer->SetRightImageColor(lut);
    lut->Delete();
}
    
void MultipleFileViewerDialog::SetViewer(DoubleWindowFileViewer *viewer)
{
    m_viewer = viewer;
}

void MultipleFileViewerDialog::SetLastVisitedDirectory(const QString & dir)
{
    m_lastVisitedDirectory = dir;
    if( !QFile::exists( m_lastVisitedDirectory ) )
    {
        m_lastVisitedDirectory = QDir::homeDirPath();
    }
}

void MultipleFileViewerDialog::SetSceneManager( SceneManager *manager )
{
    if (m_manager != manager)                                       
    {                                                           
        if (m_manager != NULL) 
        { 
            m_manager->UnRegister(0); 
        }   
        m_manager = manager;                                          
        if (m_manager != NULL) 
        { 
            m_manager->Register(0); 
        }     
    }                                                           
}

void MultipleFileViewerDialog::Initialize()
{
    int i;
    if (m_manager != NULL) 
    {
        for (i = 0; i < m_manager->GetNumberOfTemplateLokupTables(); i++)
        {
            leftColorComboBox->insertItem(m_manager->GetTemplateLookupTableName(i));
            rightColorComboBox->insertItem(m_manager->GetTemplateLookupTableName(i));
        }
        
//        leftColorComboBox->setCurrentItem(m_imageObject->GetLutIndex());     
    }
    for (i = SHORTEST_TIME_INTERVAL ; i <= LONGEST_TIME_INTERVAL; i++)
        timeComboBox->insertItem(QString::number(i));
    timeComboBox->setCurrentItem(DEFAULT_TIME_INTERVAL-SHORTEST_TIME_INTERVAL);        
}

void MultipleFileViewerDialog::SetButtonsEnabled(bool enable)
{
    firstPushButton->setEnabled(enable);
    previousPushButton->setEnabled(enable);
    nextPushButton->setEnabled(enable);
    lastPushButton->setEnabled(enable);
    deletePushButton->setEnabled(enable);
    loadVolumePushButton->setEnabled(enable);
    timeComboBox->setEnabled(enable);
    rightWindowSlider->setEnabled(enable);
    rightLevelSlider->setEnabled(enable);
    rightColorComboBox->setEnabled(enable);
    loadFilesPushButton->setEnabled(enable);
    setTransformPushButton->setEnabled(enable);
    leftWindowSlider->setEnabled(enable);
    leftLevelSlider->setEnabled(enable);
    leftLevelSlider->setEnabled(enable);
    leftColorComboBox->setEnabled(enable);
    infoPushButton->setEnabled(enable);
}
    
void MultipleFileViewerDialog::CleanAcquisition()
{
    AcquisitionDataList::iterator it = m_acquisition.begin();
    AcquisitionDataList::iterator itEnd = m_acquisition.end();
    while( it != itEnd )
    {
        (*it)->Delete();
        ++it;
    }
    m_acquisition.clear();
    m_acquisitionImageCounter = 0;
}

void MultipleFileViewerDialog::ShowMincInfo()
{
    std::ostrstream info;
    QString  filename = m_acquisition[m_current_index]->GetFileName();
    vtkMINCImageAttributes *attributes;
    vtkMINCImageReader * reader = vtkMINCImageReader::New();
    if( reader->CanReadFile( filename ) )
    {
        reader->SetFileName(filename);
        reader->Update();
        attributes = reader->GetImageAttributes();
        attributes->PrintFileHeader(info);
        QWidget *parent = this->parentWidget(true);
        if (parent)
            parent = (this->parentWidget(true))->parentWidget(true);
        MincInfoWidget *infoWidget = new MincInfoWidget(parent, "MincInfo", Qt::WDestructiveClose);
        infoWidget->SetInfoText(info.str());
        infoWidget->show();
    }
}

void MultipleFileViewerDialog::ComputeResliceTransform(int index, vtkTransform *outTransform)
{
    vtkMatrix4x4 * rotation = vtkMatrix4x4::New();

    double step[3], start[3];
    m_acquisition[index]->GetStep(&step[0], &step[1], &step[2]);
    m_acquisition[index]->GetStartCoordinates(&start[0], &start[1], &start[2]);
    vtkMatrix4x4 *dirCos = m_acquisition[index]->GetDirectionCosines();
    for( int i = 0; i < 3; i++ )
    {
        for( int j = 0; j < 3; j++ )
        {
            rotation->SetElement( j, i, dirCos->Element[i][j] );
        }
    }

    outTransform->Identity();
    outTransform->Concatenate( rotation );
    outTransform->Scale( step );
    outTransform->Translate( step );
    rotation->Delete();
}