#include "imagetracer.h"
#include "vtkRenderer.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkCommand.h"
#include "vtkCoordinate.h"
#include "vtkViewport.h"
#include "vtkMultiImagePlaneWidget.h"
#include "vtkImageData.h"
#include "vtkLinearTransform.h"
#include "vtkTransform.h"
#include "view.h"

#include "imageobject.h"
#include "scenemanager.h"

ImageTracer::ImageTracer()
{
    imageObj = 0;
    manager = 0;
}

ImageTracer::~ImageTracer()
{
    int i;
    if( this->imageObj )
    {
        this->imageObj->UnRegister( this );
    }
    if (this->manager != NULL) 
    { 
        this->manager->UnRegister(this); 
    }   
}

void ImageTracer::SetImageObject( ImageObject * obj )
{
    if( this->imageObj == obj )
    {
        return;
    }
    
    if( this->imageObj )
    {
        this->imageObj->UnRegister( this );
    }
    this->imageObj = obj;
    if( this->imageObj )
    {
        this->imageObj->Register( this );
    }
    for (int i = 0; i < 3; i++)
    {
        this->viewPlanes[i] = this->imageObj->GetPlane(i);
        initialPlanesPosition[i] = this->viewPlanes[i]->GetSlicePosition();
    }  
    if (this->manager != NULL) 
    { 
        this->manager->UnRegister(this); 
    }   
    this->manager = this->imageObj->GetManager();                                          
    if (this->manager != NULL) 
    { 
        this->manager->Register(this); 
    }  
}
   
void ImageTracer::SetTracerPosition( vtkReal *pt)
{
    if (pt)
    {
        vtkReal *pos;
        vtkTransform *t = this->imageObj->GetCurrentTransform();
        if (!t)
        {
            pos = pt;
        }
        else
        {
            vtkLinearTransform *transform = t->GetLinearInverse();
            pos = (vtkReal*)transform->TransformDoublePoint((double)pt[0], (double)pt[1], (double)pt[2]); 
        }
        this->viewPlanes[0]->SetSlicePosition(pos[0]);
        this->viewPlanes[1]->SetSlicePosition(pos[1]);
        this->viewPlanes[2]->SetSlicePosition(pos[2]);
        for (int j = 0; j < 3; j++)
        {
            View * v = this->manager->GetView(j);
            vtkCoordinate* coord = vtkCoordinate::New();
            coord->SetCoordinateSystemToWorld();
            coord->SetValue(pt);
            int *dispPos = coord->GetComputedDisplayValue(reinterpret_cast<vtkViewport*>(v->GetRenderer()));
            this->viewPlanes[j]->ExternalUpdateCursor(dispPos, v->GetRenderer());
            this->manager->GetView(j)->GetRenderer()->ResetCameraClippingRange();
            this->manager->GetView(j)->NotifyNeedRender();
        }
    }
}

void ImageTracer::DisableImageTracer()
{
    for (int i = 0; i < 3; i++)
    {
        this->viewPlanes[i]->ExternalActivateCursor(0);
        this->viewPlanes[i]->SetSlicePosition(initialPlanesPosition[i]);
        this->manager->GetView(i)->GetRenderer()->ResetCameraClippingRange();
        this->manager->GetView(i)->NotifyNeedRender();
    }
}

