/*=========================================================================

  Program:   Igns global tools
  Module:    $RCSfile: usacquisitioninfo.cpp,v $
  Language:  C++
  Date:      $Date: 2009-04-27 14:29:23 $
  Version:   $Revision: 1.1 $

  Copyright (c) 2007-2009 IPL, BIC, MNI, McGill, Anka
  All rights reserved.
  
=========================================================================*/
#include "usacquisitioninfo.h"

#include <stdio.h>
#include <time.h>
#include <qdir.h>
#include <qfiledialog.h>
#include "ignsconfig.h"
#include "version.h"
#include "vtkMINCImageAttributes.h"
#include "vtkMINC.h"
#include "vtkMINC2.h"

ObjectSerializationMacro( USAcquisitionInfo );

USAcquisitionInfo::USAcquisitionInfo() 
{
    char tmp[100];
    time_t result;
    result = time(NULL);
    this->acquisitionDate = asctime(localtime(&result));
    this->studentName = cuserid(tmp);
    this->ultrasoundModel = IGNS_ULTRASOUND_MODEL;
    this->trackerModel = IGNS_TRACKER_MODEL;
    this->application = "ibis";
    this->applicationVersion = IGNS_VERSION;
    this->lastVisitedDirectory = (const char*)(QDir::homeDirPath() + "/" + IGNS_CONFIGURATION_SUBDIRECTORY);
}

USAcquisitionInfo::~USAcquisitionInfo() 
{
}

void USAcquisitionInfo::Serialize( Serializer * serializer )
{
    ::Serialize( serializer, "PatientIdentification", patientIdentification );
    ::Serialize( serializer, "PatientName", patientName );
    ::Serialize( serializer, "StudentName", studentName );
    ::Serialize( serializer, "SurgeonName", surgeonName );
    ::Serialize( serializer, "AcquisitionDate", acquisitionDate );
    ::Serialize( serializer, "Comment", comment );
    ::Serialize( serializer, "UltrasoundModel", ultrasoundModel );
    ::Serialize( serializer, "TrackerModel", trackerModel );
    ::Serialize( serializer, "TrackerVersion", trackerVersion );
    ::Serialize( serializer, "PointerModel", pointerModel );
    ::Serialize( serializer, "PointerCalibrationFile", pointerCalibrationFile );
    ::Serialize( serializer, "PointerRomFile", pointerRomFile );
    ::Serialize( serializer, "ReferenceModel", referenceModel );
    ::Serialize( serializer, "ReferenceRomFile", referenceRomFile );
    ::Serialize( serializer, "ProbeModel", probeModel );
    ::Serialize( serializer, "ProbeCalibrationFile", probeCalibrationFile );
    ::Serialize( serializer, "ProbeRomFile", probeRomFile );
    ::Serialize( serializer, "usDepth", usDepth );
    ::Serialize( serializer, "usScale", usScale );
    ::Serialize( serializer, "application", application );
    ::Serialize( serializer, "applicationVersion", applicationVersion );
    ::Serialize( serializer, "calibrationDate", calibrationDate );
    ::Serialize( serializer, "calibrationMethod", calibrationMethod );
    ::Serialize( serializer, "calibrationVersion", calibrationVersion );
    ::Serialize( serializer, "acquisitionType", acquisitionType );
    ::Serialize( serializer, "acquisitionColor", acquisitionColor );
    ::Serialize( serializer, "lastVisitedDirectory", lastVisitedDirectory );
}

void USAcquisitionInfo::FillImageAttributes( vtkMINCImageAttributes * attributes )
{
    if (!attributes)
        return;
    // patient
    if (!patientName.isEmpty())
        attributes->SetAttributeValueAsString(MIpatient, MIfull_name, patientName);
    if (!patientIdentification.isEmpty())
        attributes->SetAttributeValueAsString(MIpatient, MIidentification, patientIdentification);
    // study
    if (!studentName.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MIoperator, studentName);
    if (!surgeonName.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MIattending_physician, surgeonName);
    attributes->SetAttributeValueAsString(MIstudy, MImodality, "Ultrasound");
    attributes->SetAttributeValueAsString(MIstudy, MImanufacturer, "ATL");
    if (!ultrasoundModel.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MIdevice_model, ultrasoundModel);
    if (!trackerModel.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MItracker_model, trackerModel);
    if (!trackerVersion.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MItracker_version, trackerVersion);
    if (!application.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MIapplication, application);
    if (!applicationVersion.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MIapplication_version, applicationVersion);
    if (!probeModel.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MItransducer_model, probeModel);
    if (!probeRomFile.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MItransducer_rom, probeRomFile);
    if (!probeCalibrationFile.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MItransducer_calibration, probeCalibrationFile);
    if (!pointerModel.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MIpointer_model, pointerModel);
    if (!pointerRomFile.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MIpointer_rom, pointerRomFile);
    if (!pointerCalibrationFile.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MIpointer_calibration, pointerCalibrationFile);
    if (!referenceModel.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MIreference_model, referenceModel);
    if (!referenceRomFile.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MIreference_rom, referenceRomFile);
    if (!calibrationMethod.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MIcalibration_method, calibrationMethod);
    if (!calibrationVersion.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MIcalibration_version, calibrationVersion);
    if (!calibrationDate.isEmpty())
        attributes->SetAttributeValueAsString(MIstudy, MIcalibration_date, calibrationDate);
    // acquisition
    if (!usDepth.isEmpty())
        attributes->SetAttributeValueAsString(MIacquisition, MIultrasound_depth, usDepth );
    if (!usScale.isEmpty())
        attributes->SetAttributeValueAsString(MIacquisition, MIultrasound_scale, usScale );
    if (!acquisitionType.isEmpty())
        attributes->SetAttributeValueAsString(MIacquisition, MIultrasound_acquisition_type, acquisitionType );
    if (!acquisitionColor.isEmpty())
        attributes->SetAttributeValueAsString(MIacquisition, MIultrasound_acquisition_color, acquisitionColor );
    if (!comment.isEmpty())
        attributes->SetAttributeValueAsString("", MIcomments, comment );
}
