//==========================================================================
//   Copyright (C) 2004 by Simon Drouin
//   sdrouin@bic.mni.mcgill.ca
//==========================================================================
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <qapplication.h>
#include <iostream>
#include "serializer.h"

using namespace std;

class Car
{

public:
    
    Car( const char * model, const char * manufact, int year )
    {
         this->model = model;
         this->manufact = manufact;
         this->year = year;
    }
    
    void Serialize( Serializer * serial )
    {
        ::Serialize( serial, "model", model );
        ::Serialize( serial, "manufact", manufact );
        ::Serialize( serial, "year", year );
        double vec[4] = { 1, 2, 3, 4 };
        ::Serialize( serial, "vec", vec, 4 );
    }
    
    int operator==( const Car & car )
    {
        if( this->model == car.model &&
            this->manufact == car.manufact &&
            this->year == car.year )
        {
            return 1;
        }
        return 0;
    }
        
private:
    
    std::string model;
    std::string manufact;
    int year;
};

struct House
{
    House() { door = 10; }
    void Serialize( Serializer * serial)
    {
        ::Serialize( serial, "Door", door );
    }        
    int door;
};

ObjectSerializationMacro( House )

ObjectSerializationMacro( Car )

int main(int argc, char *argv[])
{
  QApplication a( argc, argv );
  
  std::string writtenName( "Simon" );
  int writtenAge = 29;
  double writtenHeight = 1.72;
  Car writtenCar( "Echo", "Toyota", 1999 );
  int writtenVector[3] = { 1, 2, 3 };
  double writtenVectorD[3] = { 1.1, 2.2, 3.3 };
  std::vector<int> writtenVec;
  writtenVec.push_back(1);
  writtenVec.push_back(2);
  writtenVec.push_back(3);
  writtenVec.push_back(4);
  
  // try to write a file
  SerializerWriter writer;
  writer.SetFilename( "test.xml" );
  writer.Start();
  Serialize( &writer, "StdVector", writtenVec );
  Serialize( &writer, "name", writtenName );
  Serialize( &writer, "age", writtenAge );
  Serialize( &writer, "height", writtenHeight );
  Serialize( &writer, "car", &writtenCar );
  Serialize( &writer, "vector", writtenVector, 3 );
  Serialize( &writer, "vectorD", writtenVectorD, 3 );
  writer.Finish();
  
  std::string readName( "Simon" );
  int readAge = 29;
  double readHeight = 1.72;
  Car readCar( "", "", 0 );
  int readVector[3] = { 0, 0, 0 };
  double readVectorD[3] = { 0, 0, 0 };
  std::vector<int> readVec;
  
  // try to read a file
  SerializerReader reader;
  reader.SetFilename("test.xml");
  reader.Start();
  Serialize( &reader, "StdVector", readVec );
  Serialize( &reader, "name", readName );
  Serialize( &reader, "age", readAge );
  Serialize( &reader, "height", readHeight );
  Serialize( &reader, "car", &readCar );
  Serialize( &reader, "vector", readVector, 3 );
  Serialize( &reader, "vectorD", readVectorD, 3 );
  reader.Finish();
  
  if( readName == writtenName &&
      readAge == writtenAge &&
      readHeight == writtenHeight &&
      readCar == writtenCar &&
      readVector[0] == writtenVector[0] &&
      readVector[1] == writtenVector[1] &&
      readVector[2] == writtenVector[2] &&
      readVectorD[0] == writtenVectorD[0] &&
      readVectorD[1] == writtenVectorD[1] &&
      readVectorD[2] == writtenVectorD[2] &&
      readVec == writtenVec )
  {
      cout << "Test succeeded!" << endl;
  }
  else
  {
      cout << "Test failed!" << endl;
  }

  return EXIT_SUCCESS;
}
