#ifndef VTKMINCWRITER_H
#define VTKMINCWRITER_H

#include <stdio.h>

#include "vtkReal.h"
#include "vtkMatrix4x4.h"
#include "vtkImageData.h"

#if !defined(WIN32) && !defined(_WIN32)
// IGSTK headers
#include "igstkObject.h"
#include "igstkMacros.h"
#endif
class vtkMINCWriter : public vtkObject
{

public:

    //BTX
    char const * GetClassName() const;
    //ETX
    static vtkMINCWriter * New() { return new vtkMINCWriter; }

    
   vtkMINCWriter();
    ~vtkMINCWriter();
    
#if !defined(WIN32) && !defined(_WIN32)
    /** Set up variables, types and methods related to the Logger */
    igstkLoggerMacro()
#endif    
    void Write();

    vtkSetObjectMacro(Input,vtkImageData);
    vtkSetObjectMacro(Transform,vtkMatrix4x4);
    vtkSetStringMacro(Filename);
    
    vtkSetMacro(UseTransformForStartAndStep,int);
    vtkGetMacro(UseTransformForStartAndStep,int);

static    void vtkMatrixToStartStepAndDirCosine( vtkMatrix4x4 * mat, double start[3], double step[3], double dirCosine[3][3] );

private:

    // Description:
    // if 1, the input transform will be used to compute start
    // and step of the file. Otherwise, the start and step will be
    // taken from the input image.
    int UseTransformForStartAndStep;
    
    vtkImageData * Input;
    vtkMatrix4x4 * Transform;
    char         * Filename;

};


#endif
